PROGRAM_NAME='NVIEW, D450, RS-232, BASIC, 1-97, BA'
(*   DATE:05/20/99    TIME:15:53:40    *)
(***********************************************************)
(* The following code block is provided as a guide to      *)
(* programming the device(s) listed above. This is a       *)
(* sample only, and will most likely require modification  *)
(* to be integrated into a master program.                 *)
(*                                                         *)
(* Device-specific protocols should be obtained directly   *)
(* from the equipment manufacturer to ensure compliance    *)
(* with the most current versions. Within the limits       *)
(* imposed by any non-disclosure agreements, AMX will      *)
(* provide protocols upon request.                         *)
(*                                                         *)
(* If further programming assistance is required, please   *)
(* contact your AMX customer support team.                 *)
(*                                                         *)
(***********************************************************)
(* GENERAL FUNCTION LIST -                                 *)
(*   LAMP, PICTURE FUNCTIONS, SOURCE SELECT, VOLUME        *)
(***********************************************************)
(* NOTES -                                                 *)
(* > THIS PROJECTOR CAN BE CONTROLLED EITHER BY ITS FRONT  *)
(* PANEL BUTTONS AND IR HAND CONTROL OR BY A REMOTE DEVICE *)
(* VIA RS-232, BUT NOT BOTH AT THE SAME TIME. THE REMOTE   *)
(* DEVICE (AMX SYSTEM) DETERMINES WHO HAS CONTROL BY       *)
(* SENDING COMMANDS BOTH TO REQUEST CONTROL AND TO RETURN  *)
(* CONTROL TO THE LOCAL DEVICE. EVERY TIME THE PROJECTOR   *)
(* IS POWERED ON, IT INITIALIZES TO LOCAL CONTROL.         *)
(*                                                         *)
(* > AFTER THE "LAMP" COMMAND IS USED TO TURN THE LAMP ON, *)
(* IT REQUIRES ABOUT 5 SECONDS TO WARM UP, AND AFTER THE   *)
(* LAMP IS TURNED OFF IT REQUIRES ABOUT 3-1/2 MINUTES TO   *)
(* COOL DOWN. WHILE THE LAMP IS WARMING UP OR COOLING DOWN,*)
(* THE PROJECTOR WILL NOT ACCEPT ANY NEW COMMANDS. IT WILL *)
(* THOUGH RETURN THE STRING 'LAMP' UPON COMPLETION OF THE  *)
(* WARM-UP/COOL-DOWN PERIOD, SO THAT THE REMOTE DEVICE     *)
(* CAN KNOW EXACTLY WHEN THE PROJECTOR IS READY TO ACCEPT  *)
(* MORE COMMANDS.                                          *)
(*                                                         *)
(* > THE LAMP, MUTE, CURTAIN, IMAGE, SOUND, SETUP, AND     *)
(* POSITION COMMANDS ARE ALL TOGGLING FUNCTIONS. ALSO,     *)
(* THESE COMMANDS WILL NORMALLY BE TURNED OFF WHEN ANOTHER *)
(* OF THESE COMMANDS OR THE VOLUME OR PICTURE COMMAND IS   *)
(* SENT TO THE PROJECTOR (DEPENDING UPON OTHER FACTORS).   *)
(*                                                         *)
(* > THE "CURTAIN" COMMAND CAUSES THE PROJECTOR TO DISPLAY *)
(* A BLUE SCREEN. NO "VIDEO MUTE" FUNCTION EXISTS.         *)
(*                                                         *)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

V_PROJ = 1                             (* NVIEW D-450 *)
PANEL  = 128                           (* AMX PANEL *)

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

VP_BUFFER[100]          (* INCOMING BUFFER *)
VP_HAVE_CONTROL         (* FLAG TO TRACK IF AMX SYSTEM IS CONTROL *)
VP_LAMP_COOL            (* FLAG TO TRACK IF THE LAMP IS COOLING/WARMING *)
VP_SOURCE               (* TRACKS THE CURRENTLY SELECTED SOURCE *)
VP_REPEAT_STRING[10]    (* STORES A STRING TO BE SENT REPEATEDLY *)
VP_REPEAT_WAIT          (* STORES THE DURATION BETWEEN REPEATED SENDS *)

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*           SUBROUTINE DEFINITIONS GO BELOW               *)
(***********************************************************)

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

CREATE_BUFFER V_PROJ,VP_BUFFER
SEND_COMMAND V_PROJ,'CHARD-200'

VP_REPEAT_WAIT = 0
OFF[VP_LAMP_COOL]

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

(*** REMOTE CONTROL OF PROJECTOR ***)
PUSH[PANEL,11]                         (* REQUEST CONTROL *)
{
  SEND_STRING V_PROJ,"27"
  WAIT 3
    SEND_STRING V_PROJ,"'UNLOCK',13"
  WAIT 5
    ON[VP_HAVE_CONTROL]
}

PUSH[PANEL,12]                         (* RELEASE CONTROL *)
{
  SEND_STRING V_PROJ,"'GO',13"
  OFF[VP_HAVE_CONTROL]
}


(*** LAMP POWER ***)
PUSH[PANEL,15]                         (* LAMP *)
  IF (NOT VP_LAMP_COOL)
  {
    SEND_STRING V_PROJ,"'LAMP',13"
    ON[VP_LAMP_COOL]

    CLEAR_BUFFER VP_BUFFER
    WAIT_UNTIL(FIND_STRING(VP_BUFFER,"'LAMP',$0D",1)) 'LAMP - RESPONSE'
    {
      OFF[VP_LAMP_COOL]
      CANCEL_WAIT 'LAMP - TIMED'
    }
    WAIT 2400 'LAMP - TIMED'
    {
      OFF[VP_LAMP_COOL]
      CANCEL_WAIT_UNTIL 'LAMP - RESPONSE'
    }
  }


(*** SOURCE SELECTION ***)
PUSH[PANEL,16]                         (* SOURCE 1 *)
{
  SEND_STRING V_PROJ,"'SOURCE 1',13"
  VP_SOURCE = 1
}
[PANEL,16] = (VP_SOURCE = 1)

PUSH[PANEL,17]                         (* SOURCE 1 *)
{
  SEND_STRING V_PROJ,"'SOURCE 2',13"
  VP_SOURCE = 2
}
[PANEL,17] = (VP_SOURCE = 2)

PUSH[PANEL,18]                         (* SOURCE 1 *)
{
  SEND_STRING V_PROJ,"'SOURCE 3',13"
  VP_SOURCE = 3
}
[PANEL,18] = (VP_SOURCE = 3)

PUSH[PANEL,19]                         (* SOURCE 1 *)
{
  SEND_STRING V_PROJ,"'SOURCE 4',13"
  VP_SOURCE = 4
}
[PANEL,19] = (VP_SOURCE = 4)


(*** AUDIO FUNCTIONS ***)
PUSH[PANEL,21]                         (* VOLUME UP *)
{
  SEND_STRING V_PROJ,"'VOLUME +1',13"
  VP_REPEAT_STRING = 'VOLUME +4'
  VP_REPEAT_WAIT = 5
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,21]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,22]                         (* VOLUME DOWN *)
{
  SEND_STRING V_PROJ,"'VOLUME -1',13"
  VP_REPEAT_STRING = 'VOLUME -4'
  VP_REPEAT_WAIT = 5
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,22]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,23]                         (* AUDIO MUTE *)
  SEND_STRING V_PROJ,"'MUTE',13"


(*** PICTURE FUNCTIONS ***)
PUSH[PANEL,26]                         (* PICTURE + *)
{
  SEND_STRING V_PROJ,"'PICTURE +1',13"
  VP_REPEAT_STRING = 'PICTURE +4'
  VP_REPEAT_WAIT = 5
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,26]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,27]                         (* PICTURE - *)
{
  SEND_STRING V_PROJ,"'PICTURE -1',13"
  VP_REPEAT_STRING = 'PICTURE -4'
  VP_REPEAT_WAIT = 5
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,27]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,28]                         (* CURTAIN (BLUE SCREEN) *)
  SEND_STRING V_PROJ,"'CURTAIN',13"


(*** MENU FUNCTIONS ***)
PUSH[PANEL,31]                         (* IMAGE *)
  SEND_STRING V_PROJ,"'IMAGE',13"

PUSH[PANEL,32]                         (* SOUND *)
  SEND_STRING V_PROJ,"'SOUND',13"

PUSH[PANEL,33]                         (* SETUP *)
  SEND_STRING V_PROJ,"'SETUP',13"

PUSH[PANEL,34]                         (* POSITION *)
  SEND_STRING V_PROJ,"'POSITION',13"


(*** ARROW KEYS (FOR MENUS) ***)
PUSH[PANEL,36]                         (* UP ARROW *)
{
  SEND_STRING V_PROJ,"'UP 1',13"
  VP_REPEAT_STRING = "'UP 1',13"
  VP_REPEAT_WAIT = 7
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,36]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,37]                         (* DOWN ARROW *)
{
  SEND_STRING V_PROJ,"'DOWN 1',13"
  VP_REPEAT_STRING = "'DOWN 1',13"
  VP_REPEAT_WAIT = 7
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,37]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,38]                         (* LEFT ARROW *)
{
  SEND_STRING V_PROJ,"'LEFT 1',13"
  VP_REPEAT_STRING = "'LEFT 1',13"
  VP_REPEAT_WAIT = 3
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,38]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}

PUSH[PANEL,39]                         (* RIGHT ARROW *)
{
  SEND_STRING V_PROJ,"'RIGHT 1',13"
  VP_REPEAT_STRING = "'RIGHT 1',13"
  VP_REPEAT_WAIT = 3
  TO[PANEL,PUSH_CHANNEL]
}
RELEASE[PANEL,39]
{
  CANCEL_WAIT 'SEND REPEAT'
  VP_REPEAT_STRING = ""
  VP_REPEAT_WAIT = 0
}


(*** CHECK FOR ANY STRINGS TO BE SENT REPEATEDLY ***)
IF (VP_REPEAT_WAIT)
  WAIT VP_REPEAT_WAIT 'SEND REPEAT'
    SEND_STRING V_PROJ,"VP_REPEAT_STRING"



(********************** CODE EXAMPLE ***********************)
(* IF YOU WANT FOR THE PROJECTOR AND THE AMX SYSTEM TO     *)
(*   BOTH HAVE CONTROL OF THE PROJECTOR'S FUNCTIONS AS     *)
(*   NEEDED, THEN THE FOLLOWING EXAMPLE DISPLAYS HOW YOU   *)
(*   CAN COMBINE ANY FUNCTION WITH THE PROPER COMMANDS FOR *)
(*   REQUESTING AND RELEASING REMOTE CONTROL. WITH THIS    *)
(*   METHOD IT WILL NOT BE NECESSARY TO PERFORM 3 BUTTON   *)
(*   PUSHES TO ACCOMPLISH EVERY FUNCTION (1 TO GET CONTROL,*)
(*   1 FOR THE FUNCTION, AND 1 TO END CONTROL). THIS IS    *)
(*   DONE BY USING A FLAG VARIABLE ("VP_HAVE_CONTROL") TO  *)
(*   TRACK WHETHER OR NOT THE AMX SYSTEM IS CURRENTLY      *)
(*   CONTROLLING THE PROJECTOR, AND THEN ASSOCIATING THIS  *)
(*   FLAG WITH AN "INACTIVE" PERIOD TIMEOUT, AS FOLLOWS.   *)
(* ANY TIME A BUTTON IS PUSHED, THIS VARIABLE WILL FIRST   *)
(*   BE CHECKED, AND IF THE AMX SYSTEM IS NOT CONTROLLING  *)
(*   THE PROJECTOR, THEN THE APPROPRIATE COMMANDS WILL BE  *)
(*   SENT TO GAIN CONTROL. AT THAT TIME, A COUNTDOWN TIMER *)
(*   WILL BE STARTED (IMPLEMENTED WITH A "WAIT" STATMENT). *)
(*   IF NO SUBSEQUENT BUTTON PUSHES OCCUR BEFORE THE       *)
(*   TIMEOUT PERIOD ENDS, THEN COMMANDS WILL BE SENT TO    *)
(*   RETURN CONTROL TO THE PROJECTOR. OTHERWISE, ANY OTHER *)
(*   BUTTON PUSH WILL RESET THE TIMER.                     *)
(* THIS EXAMPLE USES A TIMEOUT PERIOD OF 10 SECONDS ("WAIT *)
(*   100") BUT THIS VALUE CAN BE CHANGED TO WHATEVER       *)
(*   TIMESPAN IS NEEDED.                                   *)
(***********************************************************)
PUSH[PANEL,100]                        (* AUDIO MUTE EXAMPLE *)
  IF (VP_HAVE_CONTROL)
  {
    SEND_STRING V_PROJ,"'MUTE',13"     (* SEND THE "MUTE" COMMAND *)
    CANCEL_WAIT 'RELEASE CONTROL'      (* STOP CURRENT 10-SECOND TIMEOUT *)
    WAIT 100 'RELEASE CONTROL'         (* BEGIN A NEW 10-SECOND TIMEOUT *)
    {
      SEND_STRING V_PROJ,"'GO',13"
      OFF[VP_HAVE_CONTROL]
    }
  }
  ELSE
  {
    SEND_STRING V_PROJ,"27"            (* REQUEST CONTROL *)
    WAIT 3
      SEND_STRING V_PROJ,"'UNLOCK',13"
    WAIT 5
    {
      ON[VP_HAVE_CONTROL]
      SEND_STRING V_PROJ,"'MUTE',13"   (* SEND THE "MUTE" COMMAND *)
    }
    WAIT 100 'RELEASE CONTROL'         (* BEGIN 10-SECOND TIMEOUT *)
    {
      SEND_STRING V_PROJ,"'GO',13"
      OFF[VP_HAVE_CONTROL]
    }
  }


(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)

